@@ -2,6 +2,7 @@
 utils
 """
 from __future__ import print_function, unicode_literals
+import inspect
 import warnings
 import sys
 import shlex
@@ -12,8 +13,10 @@ import platform
 
 
 DEBUG = bool(os.environ.get("SETUPTOOLS_SCM_DEBUG"))
-IS_WINDOWS = platform.system() == 'Windows'
+IS_WINDOWS = platform.system() == "Windows"
 PY2 = sys.version_info < (3,)
+PY3 = sys.version_info > (3,)
+string_types = (str,) if PY3 else (str, unicode)  # noqa
 
 
 def trace(*k):
@@ -26,7 +29,7 @@ def ensure_stripped_str(str_or_bytes):
     if isinstance(str_or_bytes, str):
         return str_or_bytes.strip()
     else:
-        return str_or_bytes.decode('utf-8', 'surrogateescape').strip()
+        return str_or_bytes.decode("utf-8", "surrogateescape").strip()
 
 
 def _always_strings(env_dict):
@@ -35,10 +38,7 @@ def _always_strings(env_dict):
     and not unicode.
     """
     if IS_WINDOWS or PY2:
-        env_dict.update(
-            (key, str(value))
-            for (key, value) in env_dict.items()
-        )
+        env_dict.update((key, str(value)) for (key, value) in env_dict.items())
     return env_dict
 
 
@@ -49,33 +49,35 @@ def _popen_pipes(cmd, cwd):
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
         cwd=str(cwd),
-        env=_always_strings(dict(
-            os.environ,
-            # try to disable i18n
-            LC_ALL='C',
-            LANGUAGE='',
-            HGPLAIN='1',
-        ))
+        env=_always_strings(
+            dict(
+                os.environ,
+                # try to disable i18n
+                LC_ALL="C",
+                LANGUAGE="",
+                HGPLAIN="1",
+            )
+        ),
     )
 
 
-def do_ex(cmd, cwd='.'):
-    trace('cmd', repr(cmd))
+def do_ex(cmd, cwd="."):
+    trace("cmd", repr(cmd))
     if os.name == "posix" and not isinstance(cmd, (list, tuple)):
         cmd = shlex.split(cmd)
 
     p = _popen_pipes(cmd, cwd)
     out, err = p.communicate()
     if out:
-        trace('out', repr(out))
+        trace("out", repr(out))
     if err:
-        trace('err', repr(err))
+        trace("err", repr(err))
     if p.returncode:
-        trace('ret', p.returncode)
+        trace("ret", p.returncode)
     return ensure_stripped_str(out), ensure_stripped_str(err), p.returncode
 
 
-def do(cmd, cwd='.'):
+def do(cmd, cwd="."):
     out, err, ret = do_ex(cmd, cwd)
     if ret:
         print(err)
@@ -83,21 +85,30 @@ def do(cmd, cwd='.'):
 
 
 def data_from_mime(path):
-    with io.open(path, encoding='utf-8') as fp:
+    with io.open(path, encoding="utf-8") as fp:
         content = fp.read()
-    trace('content', repr(content))
+    trace("content", repr(content))
     # the complex conditions come from reading pseudo-mime-messages
-    data = dict(
-        x.split(': ', 1)
-        for x in content.splitlines()
-        if ': ' in x)
-    trace('data', data)
+    data = dict(x.split(": ", 1) for x in content.splitlines() if ": " in x)
+    trace("data", data)
     return data
 
 
+def function_has_arg(fn, argname):
+    assert inspect.isfunction(fn)
+
+    if PY2:
+        argspec = inspect.getargspec(fn).args
+    else:
+
+        argspec = inspect.signature(fn).parameters
+
+    return argname in argspec
+
+
 def has_command(name):
     try:
-        p = _popen_pipes([name, 'help'], '.')
+        p = _popen_pipes([name, "help"], ".")
     except OSError:
         trace(*sys.exc_info())
         res = False
